#ifndef GUARD_mainmanager_c_h
#define GUARD_mainmanager_c_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mainmanager-c.h
/// @brief Class for main program control in console.

#include <vector>
#include <string>
#include <memory>

#include <glibmm.h>

#include <termios.h>

#include "fwd.h"

#include "mainmanager.h"

//#include "preference.h"
#include "laststate.h"

class CUIPlaylist;

enum ControlKey {
    PLAY_PREV = 'B',
    PLAY_NEXT = 'F',
    PLAY_PAUSE = 'P',
    PLAY_STOP = 'S',
    QUIT = 'Q',
    VOL_INCREASE = '+',
    VOL_DECREASE = '-'
};

class CUIMainManager : public MainManager
{
public:
    /// @brief Create MainManager object. (singleton pattern)
    /// @return Pointer of created MainManager object.
    static CUIMainManager& Instance()
    {
        static CUIMainManager instance;

        return instance;
    }

    void Run();

    SaguaPreference& GetPreference();

    LastState& GetLastState();

    void DoSavePreference();

#ifdef ENABLE_HTTP
    bool GetConvertUrlToCurrentEncoding() const;
#endif

    bool OpenDevice(unsigned int bits, unsigned int channels, unsigned int sampleRate);

    bool CheckOpenDevice();

    long int GetCurrentTime();

    void Refresh();

    double GetVolumn() const;

    void SetVolumn(double volumn);

    void VolumnIncrease();

    void VolumnDecrease();

    bool InPlay();

    void Finished();

    void TimeUpdate();

    void FileInfo();

    void PushList(const std::vector<std::string>& list);

    void SetStatusbar(const Glib::ustring& text);

    void SetStatusbarText(const Glib::ustring& text);

    void UpdateStatusbar();

    void UpdateStatusbarTimeoutClear();

    void ClearStatusbar();

    void TimeoutClearStatusbar();

private:
    CUIMainManager();
    ~CUIMainManager();

    CUIMainManager(const CUIMainManager&);
    CUIMainManager& operator=(const CUIMainManager&);

    const bool isCharSetUTF8_;

    termios origTermios_;

    //SaguaPreference preference_;
    LastState lastState_;
    Libao::AudioDriver& audio_;
    std::auto_ptr<SaguaAudioBuffer> audioBuffer_;
    std::auto_ptr<Audio::Mixer> mixer_;
    std::auto_ptr<PlayController> playController_;

    std::auto_ptr<CUIPlaylist> playlist_;

    Glib::RefPtr<Glib::MainLoop> mainLoop_;

    void DoSetDriver();

    void DoUnpause();

    void DoUpdateItemRead();

    void DoUpdatePlay();

    void DoUpdateStop();

    void DoUpdateItemRemoved();

    bool ReadKey(Glib::IOCondition io);

    void PrintVolumn();

    void RestorePrevTermAttr();
    void SetTermAttr();
};

#endif
