#ifndef GUARD_playlist_c_h
#define GUARD_playlist_c_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file playlist-c.h
/// @brief Class for console playlist header file.

#include <string>
#include <cstring>
#include <vector>
#include <map>

#include <memory>

#include "fwd.h"
#include "misc.h"
#include "enums.h"

#include "playlist.h"

class Tag;
class TagFactory;

class CUIPlaylist : public GetItem, public PlaylistDisplay
{
public:
    virtual ~CUIPlaylist() {}

    void Add(const std::vector<std::string>& files);

    void ShowCurrentFileInfo() const;

private:
    virtual void DoAdd(const std::vector<std::string>& files) = 0;

    virtual void DoShowCurrentFileInfo() const = 0;
};

inline void CUIPlaylist::Add(const std::vector<std::string>& files)
{
    DoAdd(files);
}

inline void CUIPlaylist::ShowCurrentFileInfo() const
{
    DoShowCurrentFileInfo();
}

class CUIPlaylistImpl : public CUIPlaylist
{
    friend class ListFile;
public:
    CUIPlaylistImpl();

private:
    CUIPlaylistImpl(const CUIPlaylistImpl&);
    CUIPlaylistImpl& operator=(const CUIPlaylistImpl&);

    //const std::auto_ptr<ListFactory> listFactory_;

    std::vector<std::string> list_;

    std::vector<std::string>::const_iterator pos_;

    std::auto_ptr<Tag> currentTagParser_;
    TagFactory& tagFactory_;

    std::string DoGetCurrent(Sagua::ListState& listState);
    std::string DoGetNext(Sagua::ListState& listState);
    std::string DoGetPrev(Sagua::ListState& listState);
    std::string DoGetFirst(Sagua::ListState& listState);
    std::string DoGetLast(Sagua::ListState& listState);
    std::string DoGetRandom(Sagua::ListState& listState);

    Glib::ustring DoCurrentItemTitle();

    Glib::ustring DoCurrentItemTime();

    unsigned int DoCurrentItemTimeSeconds();

    void DoAdd(const std::vector<std::string>& files);

    void DoShowCurrentFileInfo() const;

    std::string NormalFilename(Sagua::ListState& listState);

    std::string EmptyNullFilename(Sagua::ListState& listState)
    {
        listState = Sagua::LIST_EMPTY;

        return "";
    }

    std::string FirstNullFilename(Sagua::ListState& listState)
    {
        listState = Sagua::LIST_FIRST;

        return "";
    }

    std::string LastNullFilename(Sagua::ListState& listState)
    {
        listState = Sagua::LIST_LAST;

        return "";
    }

    std::string CheckEmptyList(Sagua::ListState& listState)
    {
        return list_.empty() ? EmptyNullFilename(listState) : LastNullFilename(listState);
    }

    /*
    void AddFile(const std::string& filePath, const bool& addListFile);
    void AddFile(const std::string& filePath)
    {
        Gtk::TreeModel::Row row( *(treeModel_->append()) );
        row[columns_.tagParsed_] = false;
        row[columns_.id_] = ++count_;
        row[columns_.filename_] = filePath;
        row[columns_.timeSeconds_] = 1;
    }

    void AddFile(const std::string& filePath,
                 const Glib::ustring& title,
                 const Glib::ustring& time,
                 const unsigned int& seconds)
    {
        Gtk::TreeModel::Row row( *(treeModel_->append()) );
        row[columns_.tagParsed_] = true;
        row[columns_.id_] = ++count_;
        row[columns_.filename_] = filePath;
        // need convert to utf-8?
        row[columns_.title_] = filePath;
        // convert from time seconds ?
        row[columns_.time_] = time;
        row[columns_.timeSeconds_] = seconds;
    }

#ifdef ENABLE_HTTP

    void GetListFromUrl(const std::string& destUrl,
                        const std::string& listFile);

#endif

    void RegisterTagFactory();

    void RegisterListType();

    bool IsListType(const std::string& filePath);
    */
};

/*
class ListFile
{
public:
    virtual void Read(const std::string& filename, PlaylistImpl& output) = 0;
    virtual void Write(const PlaylistImpl& input, const std::string& filename) = 0;

protected:
    void AddFile(const std::string& filePath, PlaylistImpl& playlist)
    {
        playlist.AddFile(filePath, false);
    }

#ifdef ENABLE_HTTP

    void AddUrl(const std::string& filePath, PlaylistImpl& playlist)
    {
        playlist.AddUrl(filePath);
    }

#endif

    Gtk::TreeModel::iterator GetFirstIter(const PlaylistImpl& playlist) const
    {
        return playlist.GetFirstIter();
    }

    void GetRawInformation(const PlaylistImpl& playlist, const Gtk::TreeModel::iterator& iter, unsigned int& seconds, Glib::ustring& title, std::string& filename) const
    {
        playlist.GetRawInformation(iter, seconds, title, filename);
    }
};

class M3UList : public ListFile
{
public:
    virtual void Read(const std::string& filename, PlaylistImpl& output);
    virtual void Write(const PlaylistImpl& input, const std::string& filename);
};

class PLSList : public ListFile
{
public:
    virtual void Read(const std::string& filename, PlaylistImpl& output);
    virtual void Write(const PlaylistImpl& input, const std::string& filename);
};

std::auto_ptr<ListFile> CreateM3UList();
std::auto_ptr<ListFile> CreatePLSList();

class ListFactory
{
public:
    /// @brief Register id to callback map.
    /// @param id Identification string 
    /// @param creator callback to produce object
    /// @return Success of register.
    bool Register(const char* const id, std::auto_ptr<ListFile> (*creator)())
    {
        return callbacks_.insert( callback::value_type(id, creator) ).second;
    }

    /// @brief Unregister id from callback map.
    /// @param id Identification string.
    /// @return Success of unregister.
    bool Unregister(const std::string& id)
    {
        return callbacks_.erase(id) == 1;
    }

    /// @brief Check factory has filename's type.
    /// @param filename Filename to check.
    /// @return Return true if factory has filename's type.
    bool Exist(const std::string& filename) const
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));

        if(i != callbacks_.end())
            return true;
        else
            return false;	
    }

    /// @brief Create ListFile object.
    /// @param filename Filename to play.
    /// @return ListFile object.
    std::auto_ptr<ListFile> Create(const std::string& filename)
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));

        if(i != callbacks_.end())
            return (i->second)();
        else
            return std::auto_ptr<ListFile>();	
    }

    bool Empty() const
    {
        return callbacks_.empty();
    }

private:
    typedef std::map<std::string, std::auto_ptr<ListFile> (*)()> callback;
    callback callbacks_;	    ///< callback map
};

*/

#endif
