#ifndef GUARD_tag_h
#define GUARD_tag_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag.h
/// @brief Abstract class for tag processor.

#include <glibmm.h>

/// @brief Abstract class for tag processor.
class Tag 
{ 
public:
    /// @brief Constructor of Tag.
    ///
    /// Gtk::Scale's minimum value is 0 and maximum value must be bigger
    /// than minimum value. So i'll take 1 to totalTimeSecond initial value
    /// , not 0.
    Tag(): totalTimeSeconds_(1) {}
    virtual ~Tag() {}

    /// @brief Get string of title section.
    const Glib::ustring& GetTitle()
    {
        return title_;
    }
	
    /// @brief Get string of album section.
    const Glib::ustring& GetAlbum()
    {
        return album_;
    }

    /// @brief Get string of genre section.
    const Glib::ustring& GetGenre()
    {
        return genre_;
    }
	
    /// @brief Get string of artist section.
    const Glib::ustring& GetArtist()
    {
        return artist_;
    }
	
    /// @brief Get string of comment section.
    const Glib::ustring& GetComment() 
    {
        return comment_;	
    }

    /// @brief Get string of year section.
    const Glib::ustring& GetYear() 
    {
        return year_;
    }
	
    /// @brief Get string of track section.
    const Glib::ustring& GetTrack() 
    {
        return track_;
    }

    /// @brief Get string of total time.
    const Glib::ustring& GetTotalTime()
    {
        return totalTime_;
    }

    /// @brief Get unsigned integer value of total time in second.
    const unsigned long& GetTotalTimeSeconds()
    {
        return totalTimeSeconds_;	
    }

    void ProcessAll()
    {
        ProcessYear();
        ProcessTrack();
        ProcessAlbum();
        ProcessGenre();
        ProcessArtist();
        ProcessComment();
    }

protected:
    // To display in window, i'll use Glib::ustring instead std::string
    Glib::ustring title_, artist_, album_, track_, year_;
    Glib::ustring genre_, comment_;
    Glib::ustring totalTime_;
    unsigned long totalTimeSeconds_;

    virtual void ProcessYear() = 0;
    virtual void ProcessTrack() = 0;
    virtual void ProcessAlbum() = 0;
    virtual void ProcessGenre() = 0;
    virtual void ProcessArtist() = 0;
    virtual void ProcessComment() = 0;
};

#endif
