/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file playlist-c.cc
/// @brief Class for console playlist source file.

#include <cstdio>

#include "../include/playlist-c.h"

#include "../include/tag.h"

#include "../include/TagFactory.h"

CUIPlaylistImpl::CUIPlaylistImpl()
    : pos_(list_.end()),
      tagFactory_(TagFactory::Instance())
{}

std::string CUIPlaylistImpl::DoGetCurrent(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    assert(pos_ != list_.end());

    return NormalFilename(listState);
}

std::string CUIPlaylistImpl::DoGetNext(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    assert(pos_ != list_.end());

    ++pos_;

    if(pos_ == list_.end())
        return LastNullFilename(listState);

    return NormalFilename(listState);
}

std::string CUIPlaylistImpl::DoGetPrev(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    assert(pos_ != list_.end());

    if(pos_ == list_.begin())
        return FirstNullFilename(listState);

    --pos_;

    return NormalFilename(listState);
}

std::string CUIPlaylistImpl::DoGetFirst(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    pos_ = list_.begin();

    return NormalFilename(listState);
}

std::string CUIPlaylistImpl::DoGetLast(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    pos_ = list_.end();
    --pos_;

    return NormalFilename(listState);
}

std::string CUIPlaylistImpl::DoGetRandom(Sagua::ListState& listState)
{
    if(list_.empty())
        return EmptyNullFilename(listState);

    pos_ = list_.begin();

    Glib::Rand rand;
    unsigned int randNum(rand.get_int_range(0, list_.size()));
    for(unsigned int i = 0; i != randNum; ++i)
        ++pos_;

    return NormalFilename(listState);
}

inline Glib::ustring CUIPlaylistImpl::DoCurrentItemTitle()
{
    assert(currentTagParser_.get() != 0 && !"Null tag parser pointer");

    return currentTagParser_.get() != 0 ? currentTagParser_->GetTitle() : Glib::ustring(*pos_);
}

inline Glib::ustring CUIPlaylistImpl::DoCurrentItemTime()
{
    assert(currentTagParser_.get() != 0 && !"Null tag parser pointer");

    return currentTagParser_.get() != 0 ? currentTagParser_->GetTotalTime() : "--:--";
}

inline unsigned int CUIPlaylistImpl::DoCurrentItemTimeSeconds()
{
    assert(currentTagParser_.get() != 0 && !"Null tag parser pointer");

    return currentTagParser_.get() != 0 ? currentTagParser_->GetTotalTimeSeconds() : 0;
}

void CUIPlaylistImpl::DoAdd(const std::vector<std::string>& files)
{
    // do some process ...
    list_ = files;
    pos_ = list_.begin();
}

void CUIPlaylistImpl::DoShowCurrentFileInfo() const
{
    if(currentTagParser_.get() != 0) {

        currentTagParser_->ProcessAll();

        std::printf("Artist  : %s\n", currentTagParser_->GetArtist().c_str());
        std::printf("Album   : %s\n", currentTagParser_->GetAlbum().c_str());
        std::printf("Track   : %s\n", currentTagParser_->GetTrack().c_str());
        std::printf("Year    : %s\n", currentTagParser_->GetYear().c_str());
        std::printf("Genre   : %s\n", currentTagParser_->GetGenre().c_str());
        std::printf("Comment : %s\n", currentTagParser_->GetComment().c_str());
    }
}

std::string CUIPlaylistImpl::NormalFilename(Sagua::ListState& listState)
{
    currentTagParser_ = IsReadable(*pos_) ?

        tagFactory_.Create(*pos_) :

        tagFactory_.Create("empty");

    listState = Sagua::LIST_NORMAL;

    return *pos_;
}
