// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file test_audio.cc
/// @brief Test for audio relate class and functions.

#include <boost/test/unit_test.hpp>

#include "../include/audio.h"

void FormatTest()
{
    Libao::Format testFormat;

    BOOST_CHECK(testFormat.Set(0, 0, 0));
    BOOST_CHECK(testFormat.Set(0, 1, 22050));
    BOOST_CHECK(testFormat.Set(1, 0, 1));

    BOOST_CHECK(testFormat.Set(8, 1, 22050));

    BOOST_CHECK(testFormat.GetBits() == 8);
    BOOST_CHECK(testFormat.GetChannels() == 1);
    BOOST_CHECK(testFormat.GetSampleRate() == 22050);

    Libao::SetDefault(testFormat);

    BOOST_CHECK(testFormat.GetBits() == 16);
    BOOST_CHECK(testFormat.GetChannels() == 2);
    BOOST_CHECK(testFormat.GetSampleRate() == 44100);
}

void AudioDriverTest()
{
    Libao::AudioDriver& testDriver(Libao::AudioDriver::Instance());

    BOOST_CHECK(testDriver.OpenDevice(0, 0, 0));
    BOOST_CHECK(testDriver.OpenDevice(8, 2, 44100));
    BOOST_CHECK(testDriver.OpenDevice(16, 3, 44100));
}

void AudioFunctionTest()
{
    BOOST_CHECK(Libao::IsDefault("default"));
    BOOST_CHECK(Libao::IsDefault("esd"));
    BOOST_CHECK(Libao::IsDefault(""));
}

boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
{
    boost::unit_test::test_suite* test = BOOST_TEST_SUITE("Audio relate class and functions tests");

    test->add(BOOST_TEST_CASE(&FormatTest), 3);
    test->add(BOOST_TEST_CASE(&AudioDriverTest), 3);
    test->add(BOOST_TEST_CASE(&AudioFunctionTest), 2);

    return test;
}
