// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file test_misc.cc
/// @brief Test for misc functions.

#include <string>
#include <map>

#include <boost/test/unit_test.hpp>

#include "../include/misc.h"

void StringFunctorTest()
{
    IsCharSpace space;
    BOOST_CHECK(space(' '));
    BOOST_CHECK(space('a'));

    IsCharEqual equal;
    BOOST_CHECK(equal('='));
    BOOST_CHECK(equal(' '));

    IsCharSlash slash;
    BOOST_CHECK(slash('/'));
    BOOST_CHECK(slash(' '));

    IsCharComa coma;
    BOOST_CHECK(coma('.'));
    BOOST_CHECK(coma(' '));
}

void StringFunctionTest()
{
    BOOST_CHECK(StringIsSpace("       "));
    BOOST_CHECK(StringIsSpace("a      "));

    BOOST_CHECK(GetExt("test").empty());
    BOOST_CHECK(GetExt("test.a") == "a");
    BOOST_CHECK(GetExt("test.ab") == "ab");
    BOOST_CHECK(GetExt("test.abc") == "abc");
    BOOST_CHECK(GetExt("test.testtest") == "testtest");
    BOOST_CHECK(GetExt("test.fir.sec") == "fir");
    BOOST_CHECK(GetExt("test.fir.sec") == "sec");

    StringHash hashFunctor;
    BOOST_CHECK(hashFunctor("") == 0);
    BOOST_CHECK(hashFunctor("abc") == 3014);

    BOOST_CHECK(ConvertSecondToTimeString(0) == "00:00");
    BOOST_CHECK(ConvertSecondToTimeString(3599) == "59:59");
    BOOST_CHECK(ConvertSecondToTimeString(3600) == "60:00");
    BOOST_CHECK(ConvertSecondToTimeString(3600) == "01:00:00");
    BOOST_CHECK(ConvertSecondToTimeString(3601) == "01:00:01");

    BOOST_CHECK(HasHttpProtocolString(""));
    BOOST_CHECK(HasHttpProtocolString("htt"));
    BOOST_CHECK(HasHttpProtocolString("http"));
    BOOST_CHECK(HasHttpProtocolString("http:"));
    BOOST_CHECK(HasHttpProtocolString("http:/"));
    BOOST_CHECK(HasHttpProtocolString("http://"));
    BOOST_CHECK(HasHttpProtocolString("http://a"));
    BOOST_CHECK(HasHttpProtocolString("  http://a"));
    BOOST_CHECK(HasHttpProtocolString("HtTp://a"));
    BOOST_CHECK(HasHttpProtocolString("abcd"));
    BOOST_CHECK(HasHttpProtocolString("ftp::"));

    BOOST_CHECK(HasFileProtocolString(""));
    BOOST_CHECK(HasFileProtocolString("fil"));
    BOOST_CHECK(HasFileProtocolString("file"));
    BOOST_CHECK(HasFileProtocolString("file:"));
    BOOST_CHECK(HasFileProtocolString("file:/"));
    BOOST_CHECK(HasFileProtocolString("file://"));
    BOOST_CHECK(HasFileProtocolString("file://a"));
    BOOST_CHECK(HasFileProtocolString("  file://a"));
    BOOST_CHECK(HasFileProtocolString("FilE://a"));
    BOOST_CHECK(HasFileProtocolString("abcd"));
    BOOST_CHECK(HasFileProtocolString("http::"));

    std::string testString;

    testString = "#";
    BOOST_CHECK(IsComment(testString));

    testString = "";
    BOOST_CHECK(IsComment(testString));

    testString = "#addqwpknpkvna";
    BOOST_CHECK(IsComment(testString));

    Glib::ustring testUstring;

    IntegerToString(1.2313, testUstring);
    BOOST_CHECK(testUstring == "1");

    IntegerToString(123, testUstring);
    BOOST_CHECK(testUstring == "123");

    IntegerToString(-1, testUstring);
    BOOST_CHECK(testUstring == "-1");
}

void FileFunctionTest()
{
    const std::string rootDir("/");
    const std::string homeDir(Glib::get_home_dir());

    BOOST_CHECK(IsDirectory(rootDir));
    BOOST_CHECK(IsDirectory(homeDir));

    BOOST_CHECK(IsExecutable(rootDir));
    BOOST_CHECK(IsExecutable(homeDir));

    BOOST_CHECK(IsReadable(rootDir));
    BOOST_CHECK(IsReadable(homeDir));

    BOOST_CHECK(Unlink(""));

    BOOST_CHECK_THROW(new OpenHandler("", 0), Exception::ErrnoException);
    BOOST_CHECK_THROW(new MmapHandler("", 0, 0), Exception::SaguaException);
    BOOST_CHECK_THROW(new FileHandler("", ""), Exception::ErrnoException);
}

void ConfigFunctionTest()
{
    std::map<std::string, std::string> testMap;
    testMap["string1"] = "TEST";
    testMap["string2"] = "123TEST";
    testMap["string3"] = " ";
    testMap["string4"] = "";

    testMap["bool1"] = "0";
    testMap["bool2"] = "1";
    testMap["bool3"] = "asdqwd";
    testMap["bool4"] = " ";
    testMap["bool5"] = "";

    testMap["int1"] = "1234";
    testMap["int2"] = "1.5023";
    testMap["int3"] = " ";
    testMap["int4"] = "";

    std::string testString;

    ReadConfigMap(testMap, "string1", testString);
    BOOST_CHECK(testString == "TEST");

    ReadConfigMap(testMap, "string2", testString);
    BOOST_CHECK(testString == "123TEST");

    ReadConfigMap(testMap, "string3", testString);
    BOOST_CHECK(testString == " ");

    ReadConfigMap(testMap, "string4", testString);
    BOOST_CHECK(testString == "");

    ReadConfigMap(testMap, "string5", testString);
    BOOST_CHECK(testString == "");
    // true while there is no assign

    bool testBool = false;

    ReadConfigMap(testMap, "bool1", testBool);
    BOOST_CHECK(testBool == false);

    ReadConfigMap(testMap, "bool2", testBool);
    BOOST_CHECK(testBool == true);

    ReadConfigMap(testMap, "bool3", testBool);
    BOOST_CHECK(testBool == true);
    // false while fail on strtol

    ReadConfigMap(testMap, "bool4", testBool);
    BOOST_CHECK(testBool == true);
    // false while fail on strtol

    ReadConfigMap(testMap, "bool5", testBool);
    BOOST_CHECK(testBool == true);
    // false while fail on strtol

    ReadConfigMap(testMap, "bool6", testBool);
    BOOST_CHECK(testBool == true);
    // false while there is no assign

    long int testInt;

    ReadConfigMap(testMap, "int1", testInt);
    BOOST_CHECK(testInt == 1234);

    ReadConfigMap(testMap, "int2", testInt);
    BOOST_CHECK(testInt == 1);

    ReadConfigMap(testMap, "int3", testInt);
    BOOST_CHECK(testInt == 0);

    ReadConfigMap(testMap, "int4", testInt);
    BOOST_CHECK(testInt == 0);

    ReadConfigMap(testMap, "int5", testInt);
    BOOST_CHECK(testInt == 1);
    // false while there is no assign
}

boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
{
    boost::unit_test::test_suite* test = BOOST_TEST_SUITE("Misc functions tests");

    test->add(BOOST_TEST_CASE(&StringFunctorTest), 4);
    test->add(BOOST_TEST_CASE(&StringFunctionTest), 21);
    test->add(BOOST_TEST_CASE(&FileFunctionTest), 0);
    test->add(BOOST_TEST_CASE(&ConfigFunctionTest), 5);

    return test;
}
