// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file test_net.cc
/// @brief Test for network functions.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <memory>
#include <cstring>

#include <boost/test/unit_test.hpp>

#include "../include/connector.h"

void UrlFunctionTest()
{
    using namespace NetUtil;

    BOOST_CHECK('0' == DecToHex(0));
    BOOST_CHECK('9' == DecToHex(9));
    BOOST_CHECK('A' == DecToHex(10));
    BOOST_CHECK('B' == DecToHex(11));
    BOOST_CHECK('F' == DecToHex(16)); // fail

    BOOST_CHECK(!NeedUrlEncode('a'));
    BOOST_CHECK(NeedUrlEncode(' '));
    BOOST_CHECK(NeedUrlEncode('/')); // fail
    BOOST_CHECK(!NeedUrlEncode('.'));

    BOOST_CHECK(!HasHttpProtocolString(" "));
    BOOST_CHECK(!HasHttpProtocolString("htt"));
    BOOST_CHECK(HasHttpProtocolString("http")); // fail
    BOOST_CHECK(HasHttpProtocolString("http://"));
    BOOST_CHECK(HasHttpProtocolString("HttP://  asd"));
}

void AddressClassTest()
{
    std::auto_ptr<Address> testAddress(CreateAddress("http://test.com", false));
    BOOST_CHECK(std::strcmp(testAddress->GetRawHostName(), "test.com") == 0);
    BOOST_CHECK(testAddress->GetPortNumber() == 80);
    BOOST_CHECK(std::strcmp(testAddress->GetRequest(), "/") == 0);
    BOOST_CHECK(!testAddress->HasAuthorStr());
    BOOST_CHECK(testAddress->GetAuthorStr() == 0);

    testAddress = CreateAddress(" ", false);
    BOOST_CHECK(std::strcmp(testAddress->GetRawHostName(), " ") == 0);
    BOOST_CHECK(testAddress->GetPortNumber() == 80);
    BOOST_CHECK(std::strcmp(testAddress->GetRequest(), "/") == 0);
    BOOST_CHECK(!testAddress->HasAuthorStr());
    BOOST_CHECK(testAddress->GetAuthorStr() == 0);

    testAddress = CreateAddress("test.com:8080/abcdef/ghi ", false);
    BOOST_CHECK(std::strcmp(testAddress->GetRawHostName(), "test.com") == 0);
    BOOST_CHECK(testAddress->GetPortNumber() == 8080);
    // except url encoded string...
    BOOST_CHECK(std::strcmp(testAddress->GetRequest(), "/abcdef/ghi%20") == 0);
    BOOST_CHECK(!testAddress->HasAuthorStr());
    BOOST_CHECK(testAddress->GetAuthorStr() == 0);

    testAddress = CreateAddress(" user:pass@test.com:8080/abc", false);
    BOOST_CHECK(std::strcmp(testAddress->GetRawHostName(), "test.com") == 0);
    BOOST_CHECK(testAddress->GetPortNumber() == 8080);
    BOOST_CHECK(std::strcmp(testAddress->GetRequest(), "/abc") == 0);
    BOOST_CHECK(testAddress->HasAuthorStr());
    BOOST_CHECK(std::strcmp(testAddress->GetAuthorStr(), "user:pass") == 0); // fail
}

void HttpRequestStrGeneratorClassTest()
{
    std::auto_ptr<Address> testAddress(CreateAddress("http://test.com", false));
    HttpRequestStrGenerator testGenerator(*testAddress);
    BOOST_CHECK(std::strcmp(testGenerator.GetString(),
                            "GET / HTTP/1.0\r\n" \
                            "User-Agent: " PACKAGE_NAME "/" VERSION "\r\n" \
                            "Host: test.com:80\r\n" \
                            "Accept: */*\r\n" \
                            "\r\n") == 0);

    testAddress = CreateAddress("user:pass@test.com:8000/zzza/  ", false);
    HttpRequestStrGenerator testGenerator2(*testAddress, "text/html");
    BOOST_CHECK(std::strcmp(testGenerator2.GetString(),
                            "GET /zzza/%20%20 HTTP/1.0\r\n" \
                            "User-Agent: " PACKAGE_NAME "/" VERSION "\r\n" \
                            "Host: test.com:8000\r\n" \
                            "Accept: text/html\r\n" \
                            "Authorization: Basic dXNlcjpwYXNz\r\n" \
                            "\r\n") == 0);
}

boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
{
    boost::unit_test::test_suite* test = BOOST_TEST_SUITE("Network functions tests");

    test->add(BOOST_TEST_CASE(&UrlFunctionTest), 3);
    test->add(BOOST_TEST_CASE(&AddressClassTest), 1);
    test->add(BOOST_TEST_CASE(&HttpRequestStrGeneratorClassTest), 0);

    return test;
}
